// (C) 2014 Yutaka Maeda
clear

//  C:\MyDirectory に複素表示ライブラリー"MyLib.sci"ある前提
cd C:\MyDirectory
exec('MyLib.sci', -1)

//「8×8」の変換行列を使って「スペクトル解析」的な実験をしてみましょう。
//  以下が「基底行列」です。

n=3; N=2^n;

// 表示処理用の初期化
axunit=100;
aNX=axunit*(N+1); aNY=axunit*(N+1); aNY1=axunit*(1+1);

//  ＤFＴ基底行列＜B＞を生成
for k = 1:N,
    for l =1:N, 
        B(k,l) = exp(%i*(k-1)*(l-1)*2*%pi/N)/N;
    end;
end;

//　DFT変換行列<A>を生成
//　基底行列の逆行列がDFT変換行列
A=inv(B); 

//　基底行列と逆行列(変換行列)は複素共役か確認
ErrNorm=norm(B.*N-conj(A)) 

//　基底行列を複素表示
f=figure(100); clf; f.axes_size = [aNX,aNX];
plotCmplx(B,f,'B')

//　変換行列を複素表示
f=figure(110); clf; f.axes_size = [aNX, aNY];
plotCmplx(A,f,'A')

//　時間ベクトルの設定
t=[0:N-1]/N; 



//------------S5_4-------------------
//「sin」波を入力し、その「スペクトル」を観測

R=sin(2*%pi*t)'; L='SIN(目標信号)'; //

// 以下が入力した「目標信号」の「1Hz sin波」の「複素表示」です。
f=figure(200); clf; f.axes_size = [aNX, aNY1];
plotCmplx(R,f,L, 1)
//「実数部」が「sin波」となって‘うねって’います。

// 振幅表示
figure(201); clf; bar(R)

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
f=figure(210); clf; f.axes_size = [aNX, aNY1];
plotCmplx(S,f,L, 1)
//「スペクトル」は「1Hz基底」と「-1Hz基底」の2か所のみに表われており
//「大きさ」は両方「4」。他の「基 底」は「ゼロ」です。

// 振幅表示
figure(211); clf; bar(abs(S))

//スペクトルより目標信号を再現(検算)
RR=B*S



//------------S5_5-------------------
//「cos」波を入力し、その「スペクトル」を観測

R=cos(2*%pi*t)'; L='COS'; //Cos波     
f=figure(220); clf; f.axes_size = [aNX, aNY1];
plotCmplx(R,f,L, 1);
figure(221);clf; bar(R)

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
f=figure(230); clf; f.axes_size = [aNX, aNY1];
plotCmplx(S,f,L, 1)
figure(231);clf; bar(abs(S))

//スペクトルより目標信号を再現(検算)
RR=B*S



//------------S5_6-------------------
//「のこぎり波」を入力し、その「スペクトル」を観測

R=[1:N]'; L='SAW'; //のこぎり波
f=figure(240); clf; f.axes_size = [aNX, aNY1];
plotCmplx(R,f,L, 1);
figure(241);clf; bar(R)

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
f=figure(250); clf; f.axes_size = [aNX, aNY1];
plotCmplx(S,f,L, 1);
figure(251);clf; bar(abs(S))
// DCTと比べて高周波数分部（基底４，５，６近辺）のスペクトルが大きい（周波数領域での偏りが少ない）

//スペクトルより目標信号を再現(検算)
RR=B*S



//------------S5_7-------------------
//「単位インパルス」を入力し、その「スペクトル」を観測

//先頭に「幅1 振幅1」の、以下のパルスを入力します。
R=zeros(N,1); R(1)=1; L='PLS1'; //(目標信号)

f=figure(260); clf; f.axes_size = [aNX, aNY1];
plotCmplx(R,f,L, 1);
figure(261);clf; bar(R)

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
f=figure(270); clf; f.axes_size = [aNX, aNY1];
plotCmplx(S,f,L, 1);
figure(271);clf; bar(abs(S))
// 「スペクトル」は「実数」のみで、かつ、すべて「1」となっています。 

//スペクトルより目標信号を再現(検算)
RR=B*S

//　次に入力位置を「2番目」にズラしてみます
R=zeros(N,1); R(2)=1; L='PLS2'; //(目標信号)
f=figure(280); clf; f.axes_size = [aNX, aNY1];
plotCmplx(R,f,L, 1);
figure(281);clf; bar(R)

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
f=figure(290); clf; f.axes_size = [aNX, aNY1];
plotCmplx(S,f,L, 1);
figure(291);clf; bar(abs(S))
// 　面白いことに「回転」しています。「逆回転」の「1Hz」です。 


//　参考までに「2番、3番」の位置に「パルス」を入れて「幅」が「2」の「パルス」を作ってみま しょう
R=zeros(N,1); R(2)=1;  R(3)=1; L='PLS23'; //(目標信号)
figure(300);clf; bar(R)

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
figure(310);clf; bar(abs(S))
// 　「振幅スペクトル」の形状に変化が出てきました「左右対称」のようです




//------------S5_8-------------------
//「複素信号」を入力し、その「スペクトル」を観測

R=%e^(2*%i*%pi*t'); L='EXP'; //複素入力
f=figure(320); clf; f.axes_size = [aNX, aNY1];
plotCmplx(R,f,L, 1);

//スペクトル(基底ベクトルの倍率)を算出
S=A*R;  
f=figure(330); clf; f.axes_size = [aNX, aNY1];
plotCmplx(S,f,L, 1);
// 結果は期待通り「第2番目「1Hz」の「スペクトル」が「実数」で現われました

//スペクトルより目標信号を再現(検算)
RR=B*S

