// (C) 2014 Yutaka Maeda
clear

//巡回畳み込み関数の設定。サンプリング定理の実験（おまけのです）で最後に使用。
//DFT用の畳み込みであり5-９,10章の畳み込み（線形畳み込み）と少し異なる。
function y=cconv(g,x) // g,x 同一サイズ
    szg=length(g);
    y=zeros(1,szg);
    xx=[x x]; // 巡回参照用に２周期分に拡張
    for t=1:szg,
        for u=1:szg,
            y(t)=y(t)+g(u)*xx(szg+t-u+1); // 拡張部を含めて畳み込み
        end,
    end,
endfunction;

//------------S6_4-----------------
//　　　実験開始


//「時刻0」より「4096」単位時間のシミュレーションを実施。
n=12; M=2^n; //シミュレーションサイズ M

//「δ信号」の「正規化 周波数」は「16/4096。これは「4096単位時間」を「1秒」とすると「256Hz」。
//同様にシミュレーション帯域幅」は「4096Hz」。
Ts=16; Fs=M/Ts; //Ts:擬似サンプリング間隔 
T0=256; F0=M/T0; //T0:擬似サンプリング対象信号の基本周期 
dt=(0:M-1).'; //単位時間

//δによる疑似サンプリングは対象信号の基本周期である256単位時間に対し、16倍
d=zeros(M,1);d(1:Ts:M-Ts+1)=1;   //櫛型関数生成
f=figure(100); clf; plot(dt,d);
f.axes_size = [700,300];

//係数ｃを成分とする0～6Hzの三角関数波を重ね合わせて入力信号を生成。
c=[0;1.2;1.0;0.8;0.6;0.4;0.2];  //信号成分係数ｃ　（帯域6Hz版）
//c=[0;1.2;1.0;0.8;0.6;0.4;0.2;1.4];  //信号成分係数ｃ （帯域7Hz版までＯＫ）
f0=F0*[0:length(c)-1];
sig=(sin(2*%pi*dt/M*f0))*c; // [DC,1Hz,2H,...]@基本周期
//　「c」は三角関数波の相対周波数順[0Hz 1Hz 2Hz …6Hz]に並んだ信号レベルの係数」。
// 　サンプリング対象信号は「sig」に合成され「6Hz」に帯域制限された信号となる。


recto=zeros(M,1); recto(1:T0)=1; //孤立波 Recto Window Function
R=sig.*recto; //Recto Window
f=figure(110);clf;  plot(dt,R)
f.axes_size = [700,300];

//振幅変調されたδ信号です。
x=d.*R; //Modulation
f=figure(120); clf; plot(dt,x)
f.axes_size = [700,300];

f=figure(130);clf; plot(0:T0-1,[x(1:T0) R(1:T0)]);
f.axes_size = [700,300];

//変調波に対してDFTを実行。
y=fft(x);
f=figure(140);clf;  plot(dt,abs(y))
f.axes_size = [700,300];
//周波数領域はδのサンプリング間隔に対応した256Hzの周期波（正規化周波数＝256Hz/4096Hz)になりました。

//1周期分を抜き出して拡大してみましょう。
f=figure(150);clf;  plot(0:Fs-1, abs(y(1:Fs)))
f.axes_size = [700,300];
//ゼロ詰めにより周波数の木目が細かくなり、連続周波数を擬似できています。

//等価な複素指数関数の式でも試してみましょう。
//　「δ関数」化した部分のみを抜出し「DFT」します。
//　ただし、「ゼロ詰め」した部分も含めて「256点」をDFT
Xs=x(1:Ts:M-Ts+1).'; //Sampled signal
f=figure(160);clf;  plot(0:Fs-1, Xs);
f.axes_size = [700,300];

Ys=fft(Xs);
f=figure(170);clf;  plot(0:Fs-1,abs(Ys));
f.axes_size = [700,300];
//こちらは繰り返しではなく、直接１周期分が求まりました。

//離散時間フーリエ逆変換の式は、周波数軸上の1周期分（-π/Ts～+π/Ts）
//を積分しろと言っています。
yDFT=y(1:Fs).';
xIDFT=ifft(yDFT);
f=figure(180); clf; plot2d3([0:length(xIDFT)-1],xIDFT);
f.axes_size = [700,300];
//サンプリングされた信号が再現できています。

//孤立波を周期波に変えてみます。
recto=zeros(M,1); recto(1:M/1)=1; //Full Recto Window Function
R=sig.*recto; //Recto Window
x=d.*R; //Modulation

f=figure(200);clf;  plot(dt,[x R]);
f.axes_size = [700,300];
y=fft(x);

//δ化したスペクトルを得ることが出来ました。
f=figure(210); clf; plot(dt,abs(y));
f.axes_size = [700,300];
f=figure(220); clf; plot(0:Fs-1,abs(y(1:Fs)));
f.axes_size = [700,300];

//離散化された周波数でのサンプル数は16点でしか有りません。16点をＩＤＦＴすると、
yDFT=y(1:F0:Fs).'; szDFT=Fs/F0;
xIDFT=ifft(yDFT);
f=figure(300);  clf; plot2d3([0:szDFT-1],xIDFT);
f.axes_size = [700,300];

//以下16点のデータ列を元の連続時間関数1周期分（シミュレーション時間256点）に、
//復元するためのsinc関数{=sin(ωt)/(πt), ω:カットオフ周波数} の周波数領域表現です。

//SINCF=[ones(1,szDFT/2) zeros(1,T0-szDFT) ones(1,szDFT/2) ]; //　<== 修正前
SINCF=[ones(1,szDFT/2+1) zeros(1,T0-szDFT-1) ones(1,szDFT/2) ]; //　<== 修正後

f=figure(305);clf;  plot([0:T0-1]-T0/2,SINCF([T0/2+1:T0 1:T0/2]));
f.axes_size = [700,300];
//[T0/2+1:T0 1:T0/2]は負の周波数表示の並び替え用インデックス

//17点幅の矩形関数のIDFT結果です。
SINCT=ifft(SINCF);
f=figure(310);clf;  plot([0:T0-1]-T0/2,SINCT([T0/2+1:T0 1:T0/2]));
f.axes_size = [700,300];


//理想低域通貨フィルタは[1 1 - 1 1 0 0 ... 0 0 0 1 - 1 1] の形状。 "1" の分部が通過帯域。
// "yDFT_ILPF"は16点から256点への拡張操作用バッファです。有効信号部、0Hz～8Hz、-7Hz～-1Hzを
// を左右の通過帯域部にコピーし他はゼロ詰めのまま。

//yDFT_ILPF=zeros(1,T0); yDFT_ILPF([1:szDFT/2 T0-szDFT/2+1:T0])=1;//　<== 修正前
yDFT_ILPF=zeros(1,T0); yDFT_ILPF([1:szDFT/2+1 T0-szDFT/2+2:T0])=yDFT(1:szDFT); // <== 修正後

//SINCFとの掛け算（＝時間領域でにSINCTとの畳み込み）を実施。

//yDFT_ILPF([1:szDFT/2 T0-szDFT/2+1:T0])=yDFT(1:szDFT);  //　<== 修正前
yDFT_ILPF=yDFT_ILPF .* SINCF; // <== 修正後
//ただしコピーされた有効信号部以外はゼロ詰めされているので、SINICFとの掛け算は形式的

xIDFT_SINC=ifft(yDFT_ILPF);
f=figure(330); clf; plot( [0:T0-1],[T0*xIDFT_SINC/Fs; R(1:T0).']);
f.axes_size = [700,300];

//  ------ 参考付録(おまけ)　--------
//SINCTの時間領域巡回畳み込みと比較（周波数領域でのSINCFによるBPF＋IDFTと等価）
xr=kron(xIDFT,[1 zeros(1,Ts-1)]); //櫛形変調信号（サンプリング信号）xを再現
f=figure(340); clf; plot(xr); 
f.axes_size = [700,300];

xIDFT_SINCT=cconv(SINCT.',xr); // ｘとSINCTの畳み込みより連続時間信号Rを復元
f=figure(350); clf; plot([0:length(xIDFT_SINCT)-1],[T0*xIDFT_SINCT/Fs; R(1:T0).']);
f.axes_size = [700,300];

//[注]　本実験はDFTによる少々強引な仮想実験。時間領域は周期波であることが前提。
// 従って、ここで使用する畳み込みは本プログラム先頭で定義の”巡回畳み込み”となる。
// 本来、フーリエ変換では時間制限されていないsinc関数との線形畳み込み。
// シミュレーションする場合は時間制限が必要になる。

